#ifndef MAP_UTILS_H
#define MAP_UTILS_H 1

#include <stdio.h>
#include <stdlib.h>
#include <vector3.h>

#include <gts.h>

#include "clib/colour.h"

#include "vertex_class.h"

#define POTENTIALS  1
#define CURVATURES  2
#define DISTANCES   4
#define PROPERTY_1  8
#define PROPERTY_2  16
#define VERTEXCOLOR 32
#define GLOBALCOLOR 64
#define SCALARS  128

#ifndef M_PI
#define M_PI 3.14159265
#endif

#define TODEG 180.0/M_PI
#define TORAD M_PI/180.0

#define MAXLINE 255

#define BRICK 512
#define BSIZE 8

/* map formats */
enum {
  BRIX, DSN6, APBS, GRASP, GTS, MSMS, MOLSURF, NONE
};

struct map_file{
  char *name;
  int format;
  /* drawflag: molscript drawing type  flag: GRASP header flag */
  unsigned int drawflag, flag;
  long ntris, nverts;
  double lambda;

  /* scalar field vars */
  double ori[3];
  double diff[3];
  /* number of gridpoints in each dim. */
  int len[3];
  /* truncated length (if there is a selection) */
  int offset[6];
  /* crystallographic info */
  double clen[3];
  double cangles[3];

  /* e.d. vars */
  double prod, plus, sigma;
  double f_t_c[4][4];
  double c_t_f[4][4];
  double ***data;

  /* GRASP vars */
  float rspacing;
  vector3 mid;
  float collow, colmid, colhigh;
  char *coldef;
  colour comp[100];
  /* MSMS vars */
  long nspheres;
  float density;
  float pradius;

  GtsSurface *surface;
  struct map_file *next;

  /* for mapping colors/scalars onto surface */
  struct map_file *scalars;
};
typedef struct map_file map;
typedef map *map_ptr;

extern map_ptr map1, curmap;
extern float contour_level;
extern double plus_level;

void add_map(char *);
double cost_angle (GtsEdge *);
void draw_map(char *);
void free_map_all(void);
void free_data(char *);
void get_map_plane(gdouble **, GtsCartesianGrid, guint, gpointer *);
double get_vert_scalar(map_ptr *, GtsVertex2 *);
void sum_area(GtsTriangle *, gdouble *);
gdouble gts2_surface_area(GtsSurface *);
void volume_foreach_face(GtsTriangle *, gdouble *);
gdouble gts2_surface_volume(GtsSurface *);
FILE *gfopen(char *, char *);
void map_get_low_high(vector3 *, vector3 *, vector3 *, vector3 *);
void map_set_plus(void);
void map_xform(double xform[4][4]);
void output_gtsedge(GtsSegment *, gpointer *);
void output_gtssurface(map_ptr *);
void output_gtstri(GtsTriangle *, gpointer *);
boolean remove_tri(GtsTriangle *, gpointer *);
void revert_tri(GtsTriangle *, gpointer *);
void set_coldef_file(char *);
colour set_colorvert(map_ptr *, float);
void set_drawable(GtsTriangle *, gpointer *);
void set_laplacian_lambda(void);
void set_mapcontour(void);
void set_vertex_attributes(GtsVertex2 *, gpointer *);
void set_verttype(char *);
void smooth_vertex(GtsVertex *v, gpointer *);
boolean stop_func(gdouble, guint, gpointer *);
void swap(char *, int);
void write_df3(at3d ***, int, int, int, int, int, int, int);

#endif
